#include <avr/io.h>
#include "com74hc595.h"
#include "com74hc595_lcd.h"
#include <util/delay.h>
 
unsigned char BackLightState = OFF;

/*
	Author       : Frank Link
	Date         : 26.01.2009
*/
void com74hc595_lcd_data( unsigned char d )
{
	com74hc595_setBit( LCD_RS );	
	com74hc595_lcd_Byte( d );
}
 
/*
	Author       : Frank Link
	Date         : 26.01.2009
*/
void com74hc595_lcd_nibble( unsigned char d )
{
	com74hc595_unsetBit( LCD_B4 );
	com74hc595_unsetBit( LCD_B5 );
	com74hc595_unsetBit( LCD_B6 );
	com74hc595_unsetBit( LCD_B7 );
	
	if ( d & 1<<4 ) com74hc595_setBit( LCD_B4 );
	if ( d & 1<<5 ) com74hc595_setBit( LCD_B5 );
	if ( d & 1<<6 ) com74hc595_setBit( LCD_B6 );
	if ( d & 1<<7 ) com74hc595_setBit( LCD_B7 );

	if ( BackLightState == ON ) com74hc595_unsetBit( LCD_LIGHT );
	if ( BackLightState == OFF ) com74hc595_setBit( LCD_LIGHT );

	com74hc595_out();

   	com74hc595_lcd_enable();
}

/*
	Author       : Frank Link
	Date         : 26.01.2009
*/
void com74hc595_lcd_Byte( unsigned char d )
{
 	com74hc595_lcd_nibble( d );
	com74hc595_lcd_nibble( d<<4 );
  	_delay_us( 45 );			// 45us
}

 /*
	Author       : Frank Link
	Date         : 26.01.2009
*/
void com74hc595_lcd_command( unsigned char d )
{

	com74hc595_unsetBit( LCD_RS );	
  	com74hc595_lcd_Byte( d );
  	switch( d )
	{
    	case 1:
    	case 2:
    	case 3: _delay_ms( 2 );
  	}
}
 
/*
	Author       : Frank Link
	Date         : 26.01.2009
*/
void com74hc595_lcd_enable(void)
{
	com74hc595_setBit( LCD_EN );
	com74hc595_out();
    _delay_us(20);                  
	com74hc595_unsetBit( LCD_EN );
	com74hc595_out();
}
 
/*
	Author       : Frank Link
	Date         : 26.01.2009
*/
void com74hc595_lcd_init(void)
{
	// Backlight off
	com74hc595_lcd_light_off();

	// wait to initialize the LCD-controller
	_delay_ms( 15 );

   	com74hc595_lcd_nibble(0x30);
   	_delay_ms(5);
   	com74hc595_lcd_nibble(0x30);
   	_delay_us(100);
   	com74hc595_lcd_nibble(0x30);
   	_delay_us(100);
 
   	// 4 bit mode
	com74hc595_lcd_nibble(0x20);	
  	_delay_us( 100 );


   	// 4Bit / 2 rows / 5x7
   	com74hc595_lcd_command( 0x28 );

	// LCD off
   	com74hc595_lcd_command( 0x08 ); 

	// LCD clear
   	com74hc595_lcd_clear();

   	// increment / nno scolling
   	com74hc595_lcd_command( 0x06 );

  	// LCD on / cursor off / no flash
   	com74hc595_lcd_command( 0x0C ); 
}
 
/*
	Author       : Frank Link
	Date         : 26.01.2009
*/
void com74hc595_lcd_light_on(void)
{
	com74hc595_setBit( LCD_LIGHT );
	com74hc595_out();
	BackLightState = ON;
}

/*
	Author       : Frank Link
	Date         : 26.01.2009
*/
void com74hc595_lcd_light_off(void)
{
	com74hc595_unsetBit( LCD_LIGHT );
	com74hc595_out();
	BackLightState = OFF;
}

/*
	Author       : Frank Link
	Date         : 26.01.2009
*/
void com74hc595_lcd_clear(void)
{
   com74hc595_lcd_command( CLEAR_DISPLAY );
   _delay_ms(5);
}
 
/*
	Author       : Frank Link
	Date         : 26.01.2009
*/
void com74hc595_lcd_home(void)
{
   com74hc595_lcd_command( CURSOR_HOME );
   _delay_ms(5);
}
 
/*
	Author       : Frank Link
	Date         : 26.01.2009
*/
void com74hc595_set_cursor(uint8_t x, uint8_t y)
{
  uint8_t tmp = 0x00;
 
  switch (y) {
    case 1: tmp=0x80+0x00+x; break;    // 1. row
    case 2: tmp=0x80+0x40+x; break;    // 2. row
    case 3: tmp=0x80+0x10+x; break;    // 3. row
    case 4: tmp=0x80+0x50+x; break;    // 4. row
  }
  com74hc595_lcd_command(tmp);
}
 
/*
	Author       : Frank Link
	Date         : 26.01.2009
*/
void com74hc595_lcd_string(char *data)
{
    while(*data) 
	{
        com74hc595_lcd_data(*data);
        data++;
    }
}

/*
	Author       : Frank Link
	Date         : 26.01.2009
*/
void com74hc595_lcd_CursorMovement( uint8_t c, uint8_t s )
{
	uint8_t tmp = 0x04;

	if ( c == 1 )
	{
		tmp |= 0x02;
	};
	if ( s == 1 )
	{
		tmp |= 0x01;
	};
   com74hc595_lcd_command( tmp );
   _delay_ms(5);
}

/*
	Author       : Frank Link
	Date         : 26.01.2009
*/
void com74hc595_lcd_DisplayState( uint8_t p, uint8_t c, uint8_t b )
{
	uint8_t tmp = 0x08;

	if ( b == 1 )
	{
		tmp |= 0x01;
	};
	if ( c == 1 )
	{
		tmp |= 0x02;
	};
	if ( p == 1 )
	{
		tmp |= 0x04;
	};
   com74hc595_lcd_command( tmp );
   _delay_ms(5);
}

/*
	Author       : Frank Link
	Date         : 26.01.2009
*/
void com74hc595_lcd_CursorState(uint8_t m, uint8_t s)
{
	uint8_t tmp = 0x10;

	if ( m == 1 )
	{
		tmp |= 0x08;
	};
	if ( s == 1 )
	{
		tmp |= 0x04;
	};
   com74hc595_lcd_command( tmp );
   _delay_ms(5);
}

/*
	Author       : Frank Link
	Date         : 26.01.2009
*/
void com74hc595_lcd_DisplayProperties(uint8_t b, uint8_t n, uint8_t f )
{
	uint8_t tmp = 0x20;

	if ( b == 1 )
	{
		tmp |= 0x10;
	};
	if ( n == 1 )
	{
		tmp |= 0x08;
	};
	if ( f == 1 )
	{
		tmp |= 0x04;
	};
   com74hc595_lcd_command( tmp );
   _delay_ms(5);
}
